/*
  Plain stack implementation.
  Copyright (C) 2001  Dmitry V. Levin <ldv@fandra.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdlib.h>
#include <string.h>
#include "pstack.h"

struct plain_stack
{
	size_t  allocated;
	size_t  used;
	const char **memory;
};

STATIC pstack
stack_alloc(void)
{
	return calloc(1, sizeof(struct plain_stack));
}

STATIC void
stack_free(pstack s)
{
	if (s)
	{
		free(s->memory);
		s->memory = 0;
		free(s);
	}
}

STATIC pstack
stack_push(pstack s, const char *data)
{
	if (s->used == s->allocated)
	{
		size_t  need =
			(s->allocated ? (s->allocated << 1) : 2) *
			sizeof(void *);
		void   *mem;

		if (!need)
			return 0;

		mem = realloc(s->memory, need);
		if (!mem)
			return 0;

		s->memory = (const char **) mem;
		s->allocated = need / sizeof(void *);
	}
	s->memory[s->used++] = data;
	return s;
}

STATIC const char *
stack_pop(pstack s)
{
	return (s->used) ? s->memory[--s->used] : 0;
}

STATIC const char *
stack_lookup(pstack s, const char *sample)
{
	size_t  i;

	for (i = 0; i < s->used; ++i)
		if (!strcmp(sample, s->memory[i]))
			return s->memory[i];
	return 0;
}
